/* -*- Mode: C -*- */

// ===========================================================================
//
//   This file:  D A R C . H
//
//   $Id: controller.h,v 1.5 2009/01/06 23:13:08 frgo Exp frgo $
//
// ===========================================================================

#if !defined( __DARC_H__ )
#define __DARC_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "generic.h"

  // --------------------------------------------------------------------------
  // DEFINITIONS 
  // --------------------------------------------------------------------------

#if !defined( DARC_BAUDRATE )
#define DARC_BAUDRATE                  38400 
#endif

#define DARC_MAX_WAIT_TICKS            500

#define DARC_CAT_CMD_PREFIX            "ZDARC"

#define DARC_CAT_CMD_VALUE             "VALUE"
#define DARC_CAT_CMD_ON                "ON"
#define DARC_CAT_CMD_OFF               "OFF"

#define DARC_CAT_CMD_END_CHAR          '.'
#define DARC_CAT_CMD_REQUEST_END_CHAR  '?'

#define DARC_CAT_CMD_RC_OK             "OK"
#define DARC_CAT_CMD_RC_FAIL           "FAIL"

#define DARC_NO_CMD                    NO_CMD

#define DARC_CAT_CMD_MAX_LENGTH        63
#define DARC_RECEIVE_BUFFER_LENGTH     (DARC_CAT_CMD_MAX_LENGTH + 1)
#define DARC_SEND_BUFFER_LENGTH        (DARC_CAT_CMD_MAX_LENGTH + 1)

#define DARC_RS_OK                     "OK.\r\n"
#define DARC_RS_FAIL                   "FAIL.\r\n"

  // DARC Events

#define DARC_EVENT_TYPE_FAILURE_MESSAGE    1
#define DARC_EVENT_TYPE_STATUS_CHANGE      2
#define DARC_EVENT_TYPE_BOOT               3

  // --------------------------------------------------------------------------
  // DATA TYPES 
  // --------------------------------------------------------------------------

  typedef struct _tsDARCEvent
  {
    uint32_t   nType;

    uint32_t   nFailureReasonCode;
    char      *pcStatusText;  

    char      *pcInfoText1;
    char      *pcInfoText2;

  }  tsDARCEvent, * tpsDARCEventPtr;

  // --------------------------------------------------------------------------
  // FUNCTION PROTOTYPES
  // --------------------------------------------------------------------------

  // Digital Appliance Remote Control (DARC) functions

  void         enable_darc( void );
  void         disable_darc( void );
  bool_t       use_darc( void );
  FILE *       darc_stream( void );

  void         darc_clear_receive_buffer( void );
  void         darc_clear_send_buffer( void );

  char *       darc_receive_buffer( void );

  rc_t         ensure_darc_init( void );
  rc_t         darc_init( unsigned long );
  int          darc_putc( unsigned char, FILE * );
  unsigned int darc_getc( FILE * );

  rc_t         darc_format( const char *, ... );

  rc_t         darc_send_result( rc_t );
  rc_t         darc_receive( void );

  rc_t         darc_send_event( tpsDARCEventPtr );
  void         darc_get_command( void );
  bool_t       is_darc_command_available( void );
  void         clear_darc_command_is_available( void );
  void         darc_clear_command_receiving( void );
     
#ifdef __cplusplus
} /* closing brace for extern "C" */
#endif

#endif
