/* -*- Mode: C -*- */

#include <stdio.h>
#include <string.h>
#include <stdint.h>

#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <avr/io.h>

#include "lcd.h"
#include "lcd_tools.h"

#include "generic.h"
#include "error.h"
#include "display.h"

// --------------------------------------------------------------------------
// LOCAL DEEINES                                                              
// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// Special VARS                                                              
// --------------------------------------------------------------------------

static char aacLCDBuffer[LCD_LINES][LCD_LINE_LENGTH + 1];

// --------------------------------------------------------------------------
// CLEAR_DISPLAY_LINE                                                    
// --------------------------------------------------------------------------
// Sets buffer of display line n to 0s                       
//                                                                           
// Parameters:                                                               
//   None.                                  
// --------------------------------------------------------------------------

void clear_display_line ( uint16_t nLineNr )
{
  aacLCDBuffer[ nLineNr - 1 ][ 0 ] = '\0';
  
  return;
}

// --------------------------------------------------------------------------
// CLEAR_DISPLAY                                                    
// --------------------------------------------------------------------------
// ... obvious ...                      
//                                                                           
// Parameters:                                                               
//   None.                                  
// --------------------------------------------------------------------------

void clear_display ( void )
{
  for( uint8_t nI = 1; nI <= LCD_LINES; nI++)
    clear_display_line( nI );

  lcd_clrscr();

  return;
}

// --------------------------------------------------------------------------
// DISPLAY_LINE                                                                 // --------------------------------------------------------------------------
// Display one line on LCD                                                     
//                                                                           
// Parameters:                                                               
//   None. 
// --------------------------------------------------------------------------

void display_line ( uint8_t nLineNr, const char *pcText )
{
  snprintf( aacLCDBuffer[ nLineNr - 1], 
            LCD_LINE_LENGTH + 1,
            "%s", 
            pcText );
    
  update_lcd();

  return;
}

rc_t format( FILE * stream, const char *pcFormat, ... )
{
  rc_t nRC = RC_OK;

  va_list argp;

  va_start( argp, pcFormat );
  vfprintf( stream, pcFormat, argp );
  va_end( argp ); 

  return nRC;
}

void display_format_buffer( uint8_t nLineNr, const char *pcFormat, ... )
{
  va_list ap;

  if( IS_VALID_LCD_LINE_NR(nLineNr) )
  {
    va_start(ap, pcFormat);
    vsnprintf( aacLCDBuffer[ nLineNr - 1], 
               LCD_LINE_LENGTH + 1,
               pcFormat, 
               ap );
    va_end(ap);
  }
  
  return;
}

void display_format( uint8_t nLineNr, const char *pcFormat, ... )
{
  va_list ap;
  
  if( IS_VALID_LCD_LINE_NR(nLineNr) )
  {
    va_start(ap, pcFormat);
    vsnprintf( aacLCDBuffer[ nLineNr - 1], 
               LCD_LINE_LENGTH + 1,
               pcFormat, 
               ap );
    va_end(ap);
  }

  update_lcd();
  
  return;
}

// --------------------------------------------------------------------------
// UPDATE_LCD                                                                
// --------------------------------------------------------------------------
// Clears the display and shows contents of Display Lines 1 to 4
//
// --------------------------------------------------------------------------

void update_lcd ( void )
{
  uint8_t nI;
  uint8_t nTmpSREG = SREG; 

  cli();

  lcd_clrscr();
    
  for (nI = 0; nI < LCD_LINES; nI++ )
  {
    lcd_gotoxy( nI + 1, 1 );
    lcd_puts( aacLCDBuffer[ nI ] );
  }

  SREG = nTmpSREG;

  return;
}
