#if !defined( __ERROR_H__ )
#define __ERROR_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include <string.h>
#include <stdint.h>
#include <stdlib.h>

#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/eeprom.h>
#include <util/delay.h>

#include "generic.h"
#include "eeprom.h"

// --------------------------------------------------------------------------
// DEFINES                                                                   
// --------------------------------------------------------------------------

#define ERROR_FATAL_BASE                    1000
#define ERROR_RECOVERABLE_BASE              2000
#define ERROR_WARNING_BASE                  3000
#define ERROR_EXTERNAL_BASE                 8000
#define ERROR_INTERNAL_BASE                 9000

  // Error Codes
  
#define NO_ERROR                                0
  
  // -- FATAL ERRORS (1001 - 1999)
  
#define BOOT_ERROR                           1001
#define INIT_ERROR                           1002
  
#define UNKNOWN_PSU_FAULT                    1010
  
#define I2C_ERROR                            1020
  
#define IO_TIMER_ISR_REENTRY_ERROR           1201
  
  // -- RECOVERABLE ERRORS (2001 - 2999)
  
#define EEPROM_TIMEOUT_ERROR                 2001
  
  // -- WARNINGS (3000 - 3999)
  
  // -- EXTERNAL / HANDLING ERRORS (8000 - 8999)
  
#define BROWNOUT_ILLEGAL_CONTROLLER_READ     8001
  
  // -- INTERNAL ERRORS (9000 - 9999)
  
#define SW_ERROR                             9900
#define SW_INVALID_LCD_LINE_NR_ERROR         9901
#define SW_ERROR_CANNOT_GET_LOCK             9902

#define DUMMY_ERROR                          9999 // For debugging purposes ...
  
  // -- HELPER "MACROS"
  
#define IS_FATAL_ERROR(n)       ((n >  1000) && (n <  2000))
#define IS_RECOVERABLE_ERROR(n) ((n >  2000) && (n <  3000))
#define IS_WARNING(n)           ((n >  3000) && (n <  4000))
#define IS_EXTERNAL_ERROR(n)    ((n >  8000) && (n <  9000))
#define IS_INTERNAL_ERROR(n)    ((n >  9000) && (n < 10000))
  
#define IS_ERROR(n)  ((IS_FATAL_ERROR(n)) || \
(IS_RECOVERABLE_ERROR(n)) || \
(IS_EXTERNAL_ERROR(n)) ||		 \
(IS_INTERNAL_ERROR(n))) 

#define RESET_ON_ERROR(rc)  if (IS_ERROR(rc)) { log_error(rc, __LINE__);  reset(); }
  
// --------------------------------------------------------------------------
// Function prototypes
// --------------------------------------------------------------------------

  extern void init_error_logging( void );
  extern void log_error( uint16_t, uint16_t );
  extern void get_error_log( void * );
  extern void get_last_logged_error( void );
  extern void read_nr_logged_errors( void );
  extern void incf_nr_logged_errors( void );
  extern void reset_eeprom_error_log( void );
  extern void show_last_logged_error( unsigned long );
  extern uint16_t get_nr_logged_errors( void );
  extern uint16_t get_error_nr( void );
  extern uint16_t get_error_line( void );
  extern rc_t get_logged_error( uint16_t, uint16_t *, uint16_t * );
  extern rc_t print_errorlog( FILE * );
   
#ifdef __cplusplus
} /* closing brace for extern "C" */
#endif

#endif
