/* -*- Mode: C -*- */

//   $Id$

#include <stdio.h>
#include <string.h>
#include <stdint.h>
#include <stdlib.h>
#include <avr/io.h>

#include "gaul.h"

volatile uint8_t nLCDI2CAddress; 

uint8_t i2c_lcd_read( uint8_t nAddress, uint8_t nRegister )
{
  uint8_t nValue;

  i2c_start( nAddress + I2C_WRITE );
  i2c_write( nRegister );
  i2c_rep_start( nAddress + I2C_READ );
  nValue = i2c_readAck();  
  i2c_stop(); 

  return nValue;
}

rc_t i2c_lcd_write( uint8_t nAddress, uint8_t nRegister, uint8_t nData )
{
  i2c_start( nAddress );
  i2c_write( nRegister );
  i2c_write( nData );
  i2c_stop();

  _delay_us( 100 );

  return RC_OK;
}

rc_t i2c_lcd_send_command( uint8_t nCmd )
{
  rc_t  nRC = RC_OK;

  /***
  uint8_t nFIFOFreeBytes;
  bool_t   bBusy  = TRUE;
  uint32_t nTicks = 0;

  while( bBusy == TRUE )
  {
    nFIFOFreeBytes = i2c_lcd_read( nLCDI2CAddress, I2C_LCD_REGISTER_0 );
    if( nFIFOFreeBytes >= I2C_LCD_MIN_FIFO_FREE_BYTES )
      bBusy = FALSE;
    else
    {
      nTicks++;
      if( nTicks > I2C_LCD_MAX_WAIT_TICKS )
        break;
    }
  }
 
  if( bBusy == FALSE )
  {
    i2c_lcd_write( nLCDI2CAddress, I2C_LCD_REGISTER_0, nCmd );
  }
  else
  {
    nRC = 9999;
    set_error_led();
  }

***/

  i2c_lcd_write( nLCDI2CAddress, I2C_LCD_REGISTER_0, nCmd );

  return nRC;
}

rc_t i2c_lcd_init( uint8_t nMode, uint8_t _nLCDI2CAddress )
{
  rc_t nRC = RC_OK;

  // Save address
  
  nLCDI2CAddress = _nLCDI2CAddress;

  // Set LCD 

  switch( nMode )
  {
    case LCD_DISP_OFF:
      i2c_lcd_send_command( LCD_CMD_HIDE_CURSOR );
      i2c_lcd_send_command( LCD_CMD_BACKLIGHT_OFF );
      break;

    case LCD_DISP_ON:
      i2c_lcd_send_command( LCD_CMD_HIDE_CURSOR );
      i2c_lcd_send_command( LCD_CMD_BACKLIGHT_ON );
      break;

    case LCD_DISP_ON_CURSOR:
      i2c_lcd_send_command( LCD_CMD_SHOW_UNDERLINE_CURSOR );
      i2c_lcd_send_command( LCD_CMD_BACKLIGHT_ON );
      break;

    case LCD_DISP_ON_CURSOR_BLINK:
      // i2c_lcd_send_command( LCD_CMD_SHOW_UNDERLINE_CURSOR );
      i2c_lcd_send_command( LCD_CMD_SHOW_BLINKING_CURSOR );
      i2c_lcd_send_command( LCD_CMD_BACKLIGHT_ON );
      break;
   
    default:
      break;
  }

  // Clear display

  i2c_lcd_send_command( LCD_CMD_CLEAR_SCREEN );

  // Set cursor to home position
 
  i2c_lcd_send_command( LCD_CMD_CURSOR_HOME ); 

  // Wait a bit

  _delay_ms( 250 );

  return nRC;
}

// API Functions

void i2c_lcd_clrscr( void )
{
  i2c_lcd_send_command( LCD_CMD_CLEAR_SCREEN );
}

void i2c_lcd_home( void )
{
  i2c_lcd_send_command( LCD_CMD_CURSOR_HOME ); 
}

void i2c_lcd_gotoxy( uint8_t x, uint8_t y )
{
  i2c_lcd_send_command( LCD_CMD_SET_CURSOR_XY );
  i2c_lcd_send_command( x );
  i2c_lcd_send_command( y );
}

void i2c_lcd_putc( char c )
{
  i2c_lcd_send_command( c );
}

void i2c_lcd_puts( const char *s )
{
  register char c;

  while ( (c = *s++) ) 
  {
    i2c_lcd_putc( c );
  }
}

void i2c_lcd_puts_p(const char *progmem_s)
{
  register char c;

  while ( (c = pgm_read_byte(progmem_s++)) ) 
  {
    i2c_lcd_putc(c);
  }
}
