/* -*- Mode: C -*- */

#include "i2cmaster.h"
#include "error.h"

#include "i2c_util.h"

// --------------------------------------------------------------------------
// LOCAL DEEINES                                                              
// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// Special VARS                                                              
// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// I2C_WRITE_BYTE
// --------------------------------------------------------------------------
// Writes one byte to the I2C bus
//                                                                           
// Parameters:                                                               
//   addr = device address
//   data = data byte
// --------------------------------------------------------------------------

void i2c_write_byte ( unsigned char cAddr,
                     unsigned char cData )
{
  int nRC = 0;
  
  nRC = i2c_start( cAddr + I2C_WRITE );
     
  nRC = i2c_write( cData );
  i2c_stop();

  i2c_check_error( nRC, __FILE__, __LINE__ );
     
  return;
}

// --------------------------------------------------------------------------
// I2C_READ_BYTE
// --------------------------------------------------------------------------
// Reads one byte from the I2C bus
//                 
// Parameters:                                                               
//   addr = device address
// --------------------------------------------------------------------------

unsigned char i2c_read_byte ( unsigned char cAddr )
{
  unsigned char cData = 0;
  
  i2c_start( cAddr + I2C_READ );
  
  cData = i2c_read( 0 );
  
  i2c_stop();
  
  return cData;
}

// --------------------------------------------------------------------------
// I2C_CHECK_ERROR
// --------------------------------------------------------------------------
// Checks if error (!= 0) or success (= 0)
//                                                                           
// Parameters:                                                               
//   nRC    = result of previous operation
//   pcFile = filename of file in which error occured
//   nLine  = line in which error was detected
// --------------------------------------------------------------------------

void i2c_check_error ( int nRC, char *pcFile, int nLine )
{
  if( nRC != 0 )
  {
    log_error( I2C_ERROR, nLine );
  }
  return;
}
