/* -*- Mode: C -*- */

// ===========================================================================
//
//   --->>>   G A U L   <<<---
//
//   * G oenninger & Co.
//   * A VR
//   * U tility
//   * L ibrary
//
//   This file:   L O C K . C
//                Very simple locking mechanism
//
//   $Id: controller.c,v 1.6 2009/01/06 23:12:47 frgo Exp frgo $
//
// ===========================================================================

#include <stdio.h>
#include <string.h>
#include <stdint.h>
#include <stdlib.h>

#include "lock.h"

// --------------------------------------------------------------------------
// GLOBAL DEFINITIONS
// --------------------------------------------------------------------------

// DEBUG "SWITCHES"

// --------------------------------------------------------------------------
// GLOBAL VARS
// --------------------------------------------------------------------------

static volatile uint8_t nLock = 0;

// --------------------------------------------------------------------------
// API
// --------------------------------------------------------------------------

rc_t lock ( void )
{
  uint8_t  nTmpSREG = SREG; // Save status 
  rc_t     nRC      = RC_OK;
  
  cli();                   // Ensure atomic execution
  
  if( nLock == 0 )
  {
    nLock = 1;
  }
  else 
  {
    nRC = RC_ALREADY_LOCKED;
  }
    
  SREG = nTmpSREG;         // We're done so interrupts ok now
  
  return nRC;  
}

void unlock ( void )
{
  uint8_t  nTmpSREG = SREG; // Save status 
  
  cli();                   // Ensure atomic execution
  
  nLock = 0;
  
  SREG = nTmpSREG;         // We're done so interrupts ok now
  
  return;  
}

rc_t get_lock ( void )
{
  rc_t     nRC           = RC_OK;
  bool_t   bLockAcquired = FALSE;
  uint16_t nTries        = 0;

  while(( bLockAcquired == FALSE) && ( nTries < GET_LOCK_MAX_TRIES ))
  {
    if( lock() == RC_OK )
      bLockAcquired = TRUE;
    else
      nTries++;
  }
  
  if( bLockAcquired == TRUE )
    nRC = RC_OK;
  else
    nRC = RC_BUSY;

  return nRC;  
}
